/* Copyright (C) 1979-1996 TcX AB & Monty Program KB & Detron HB

   This software is distributed with NO WARRANTY OF ANY KIND.  No author or
   distributor accepts any responsibility for the consequences of using it, or
   for whether it serves any particular purpose or works at all, unless he or
   she says so in writing.  Refer to the Free Public License (the "License")
   for full details.
   Every copy of this file must include a copy of the License, normally in a
   plain ASCII text file named PUBLIC.	The License grants you the right to
   copy, modify and redistribute this file, but only under certain conditions
   described in the License.  Among other things, the License requires that
   the copyright notice and this notice be preserved on all copies. */

#include "isamdef.h"

static void get_options(int argc, char *argv[]);

static int rec_pointer_size=0,verbose=0,remove_ant=0,pack_keys=1,flags[50],
  packed_field=FIELD_SKIPP_PRESPACE;

int main(argc,argv)
int argc;
char *argv[];
{
  N_INFO *file;
  int i,j,error,deleted,found;
  char record[128],key[32],*filename,read_record[128];
  N_KEYDEF keyinfo[10];
  N_RECINFO recinfo[10];
  MY_INIT(argv[0]);

  filename= "test1";
  my_init();
  get_options(argc,argv);
  keyinfo[0].seg[0].base.type=HA_KEYTYPE_NUM;
  keyinfo[0].seg[0].base.flag=(uint8) (pack_keys ?
				       HA_PACK_KEY | HA_SPACE_PACK : 0);
  keyinfo[0].seg[0].base.start=0;
  keyinfo[0].seg[0].base.length=6;
  keyinfo[0].seg[1].base.type=HA_KEYTYPE_END;
  keyinfo[0].base.flag = (uint8) (pack_keys ?
				  HA_NOSAME | HA_PACK_KEY : HA_NOSAME);

  recinfo[0].base.type=packed_field; recinfo[0].base.length=6;
  recinfo[1].base.type=FIELD_NORMAL; recinfo[1].base.length=24;
  recinfo[2].base.type=FIELD_LAST;

  deleted=0;
  bzero((byte*) flags,sizeof(flags));

  printf("- Creating isam-file\n");
  if (ni_create(filename,1,keyinfo,recinfo,
		(ulong) (rec_pointer_size ? (1L << (rec_pointer_size*8))/40 :
			 0),10l,0,0,0L))
    goto err;
  if (!(file=ni_open(filename,2,HA_OPEN_ABORT_IF_LOCKED)))
    goto err;
  printf("- Writing key:s\n");
  strmov(record,"      ..... key"); strappend(record,30,' ');

  my_errno=0;
  for (i=49 ; i>=1 ; i-=2 )
  {
    j=i%25 +1;
    sprintf(key,"%6d",j);
    bmove(record,key,6);
    error=ni_write(file,record);
    flags[j]=1;
    if (verbose || error)
      printf("J= %2d  ni_write: %d  errno: %d\n", j,error,my_errno);
  }
  if (ni_close(file)) goto err;
  printf("- Reopening file\n");
  if (!(file=ni_open(filename,2,HA_OPEN_ABORT_IF_LOCKED))) goto err;
  printf("- Removing keys\n");
  for (i=1 ; i<=10 ; i++)
  {
    if (i == remove_ant) { VOID(ni_close(file)) ; exit(0) ; }
    sprintf(key,"%6d",(j=(int) ((rand() & 32767)/32767.*25)));
    my_errno=0;
    if ((error = ni_rkey(file,read_record,0,key,0,HA_READ_KEY_EXACT)))
    {
      if (verbose || (flags[j] == 1 ||
		      (error && my_errno != HA_ERR_KEY_NOT_FOUND)))
	printf("key: %s  ni_rkey:   %3d  errno: %3d\n",key,error,my_errno);
    }
    else
    {
      error=ni_delete(file,read_record);
      if (verbose || error)
	printf("key: %s  ni_delete: %3d  errno: %3d\n",key,error,my_errno);
      flags[j]=0;
      if (! error)
	deleted++;
    }
  }
  printf("- Reading records with key\n");
  for (i=1 ; i<=25 ; i++)
  {
    sprintf(key,"%6d",i);
    bmove(record,key,6);
    my_errno=0;
    error=ni_rkey(file,read_record,0,key,0,HA_READ_KEY_EXACT);
    if (verbose ||
	(error == 0 && flags[i] != 1) ||
	(error && (flags[i] != 0 || my_errno != HA_ERR_KEY_NOT_FOUND)))
    {
      printf("key: %s  ni_rkey: %3d  errno: %3d  record: %s\n",
	      key,error,my_errno,record+1);
    }
  }

  printf("- Reading records with position\n");
  for (i=1,found=0 ; i <= 30 ; i++)
  {
    my_errno=0;
    if ((error=ni_rrnd(file,read_record,i == 1 ? 0L : NI_POS_ERROR)) == -1)
    {
      if (found != 25-deleted)
	printf("Found only %d of %d records\n",found,25-deleted);
      break;
    }
    if (!error)
      found++;
    if (verbose || (error != 0 && error != 1))
    {
      printf("pos: %2d  ni_rrnd: %3d  errno: %3d  record: %s\n",
	     i-1,error,my_errno,read_record+1);
    }
  }
  if (ni_close(file)) goto err;
  my_end(MY_CHECK_ERROR);

  exit(0);
err:
  printf("got error: %3d when using nisam-database\n",my_errno);
  exit(1);
  return 0;			/* skipp warning */
} /* main */


	/* l{ser optioner */
	/* OBS! intierar endast DEBUG - ingen debuggning h{r ! */

static void get_options(argc,argv)
int argc;
char *argv[];
{
  char *pos;

  while (--argc >0 && *(pos = *(++argv)) == '-' ) {
    switch(*++pos) {
    case 'R':				/* Length of record pointer */
      rec_pointer_size=atoi(++pos);
      if (rec_pointer_size > 3)
	rec_pointer_size=0;
      break;
    case 'P':
      pack_keys=0;			/* Don't use packed key */
      break;
    case 'S':
      packed_field=FIELD_NORMAL;	/* static-size record*/
      break;
    case 'v':				/* verbose */
      verbose=1;
      break;
    case 'm':
      remove_ant=atoi(++pos);
      break;
    case 'V':
      printf("isamtest1	 Ver 1.0 \n");
      exit(0);
    case '#':
      DEBUGGER_ON;
      DBUG_PUSH (++pos);
      break;
    }
  }
  return;
} /* get options */
